//============================================================================
// Name        : FractalTest.cpp
// Author      : Andrea Rossetti (andrear1979@hotmail.com)
// Version     :
// Copyright   : Free software, but no warranty.
// Description : Hello World in C, Ansi-style
//============================================================================

#include <iostream>
#include <sstream>
#include <complex>
#include <boost/gil/gil_all.hpp>
#include <boost/gil/extension/io/png_dynamic_io.hpp>

using namespace std;
using namespace boost::gil;

typedef complex<double> comp;
typedef double scal;

int main(int args, char* arg[]) {
	comp z,z_tmp,c;
	comp center(0.0, 0.0);
	scal width=4.0, height=3.0, bailout=4.0;
	int width_steps=1024, height_steps=768, iters=256;

	/*for (int i=0;i<args;i++){
		cout << "arg[" << i << "]=" << arg[i] << endl;
	}*/

	istringstream instream;
	if (args>=3){
		instream.clear();
		instream.str(arg[1]);
		instream >> width_steps;
		instream.clear();
		instream.str(arg[2]);
		instream >> height_steps;
	}

	typedef point2<ptrdiff_t>   point_t;
	point_t dims(width_steps, height_steps);
	gray8_image_t img(dims);
	gray8_view_t imgview=view(img);
	cout << "* generate mandel (" << width_steps << "x" << height_steps << ")..." << endl;

	for (int i=0;i<height_steps;i++){
        gray8_view_t::x_iterator img_row_iterator = imgview.row_begin(i);

		for (int j=0;j<width_steps;j++){

			c=comp(
					center.real()-(width/2.0)+(width/width_steps*j),
					center.imag()-(height/2.0)+(height/height_steps*i)
			);
			z=comp(0.0,0.0);
			int k;
			for (k=0 ; k<iters && (z.real()*z.real()+z.imag()*z.imag()<bailout) ; k++){
				z=z*z+c;
			}
			img_row_iterator[j]=(unsigned char)k;
		}
	}

	cout << "* write file..." << endl;

	// Save the Mandelbrot set and its 90-degree rotated gradient ; must convert to unsigned char)
	png_write_view("mandel.png",color_converted_view<gray8_pixel_t>(const_view(img)));

	cout << "end" << endl;
	return EXIT_SUCCESS;
}
